import 'package:hash/hash.dart';
import 'package:test/test.dart';

String encodeHEX(List<int> bytes) {
  var str = '';
  for (var i = 0; i < bytes.length; i++) {
    var s = bytes[i].toRadixString(16);

    str += s.padLeft(2, '0');
  }
  return str;
}

List<int> decodeHEX(String hex) {
  var bytes = <int>[];
  var len = hex.length ~/ 2;
  for (var i = 0; i < len; i++) {
    bytes.add(int.parse(hex.substring(i * 2, i * 2 + 2), radix: 16));
  }
  return bytes;
}

void main() {
  group('support md5', () {
    var valid = <String>[
      '5fa6031211db79608db137d0a63d4a940208b103',
      '5b998007b69344e6acc6e8ff052e6d27',
      'message digest',
      'f96b697d7cb7938d525a2f31aaf161d0'
    ];
    for (var i = 0; i < valid.length; i += 2) {
      test('md5: ${valid[i]}', () {
        expect(valid[i + 1],
            encodeHEX(MD5().update(valid[i].codeUnits).digest()));
      });
    }
  });

  group('support ripemd160', () {
    var valid = <String>[
      'abc',
      '8eb208f7e05d987a9b044a8e98c6b087f15a0bfc',
      'message digest',
      '5d0689ef49d2fae572b881b123a85ffa21595f36'
    ];
    for (var i = 0; i < valid.length; i += 2) {
      test('ripemd160: ${valid[i]}', () {
        expect(valid[i + 1],
            encodeHEX(RIPEMD160().update(valid[i].codeUnits).digest()));
      });
    }
  });

  group('support sha1', () {
    var valid = <String>['abc', 'a9993e364706816aba3e25717850c26c9cd0d89d'];
    for (var i = 0; i < valid.length; i += 2) {
      test('sha1: ${valid[i]}', () {
        expect(valid[i + 1],
            encodeHEX(SHA1().update(valid[i].codeUnits).digest()));
      });
    }
  });

  group('support sha224', () {
    var valid = <String>[
      'abc',
      '23097d223405d8228642a477bda255b32aadbce4bda0b3f7e36c9da7'
    ];
    for (var i = 0; i < valid.length; i += 2) {
      test('sha224: ${valid[i]}', () {
        expect(valid[i + 1],
            encodeHEX(SHA224().update(valid[i].codeUnits).digest()));
      });
    }
  });

  group('support sha256', () {
    var valid = <String>[
      'abc',
      'ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad'
    ];
    for (var i = 0; i < valid.length; i += 2) {
      test('sha256: ${valid[i]}', () {
        expect(valid[i + 1],
            encodeHEX(SHA256().update(valid[i].codeUnits).digest()));
      });
    }
  });

  group('support sha384', () {
    var valid = <String>[
      'abc',
      'cb00753f45a35e8bb5a03d699ac65007272c32ab0eded1631a8b605a43ff5bed8086072ba1e7cc2358baeca134c825a7'
    ];
    for (var i = 0; i < valid.length; i += 2) {
      test('sha384: ${valid[i]}', () {
        expect(valid[i + 1],
            encodeHEX(SHA384().update(valid[i].codeUnits).digest()));
      });
    }
  });

  group('support sha512', () {
    var valid = <String>[
      'abc',
      'ddaf35a193617abacc417349ae20413112e6fa4e89a97ea20a9eeee64b55d39a2192992a274fc1a836ba3c23a3feebbd454d4423643ce80e2a9ac94fa54ca49f'
    ];
    for (var i = 0; i < valid.length; i += 2) {
      test('sha512: ${valid[i]}', () {
        expect(valid[i + 1],
            encodeHEX(SHA512().update(valid[i].codeUnits).digest()));
      });
    }
  });
}
